@@ -1,3 +1,33 @@
+/* GDK - The GIMP Drawing Kit
+ * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
+/*
+ * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
+ * file for a list of people on the GTK+ Team.  See the ChangeLog
+ * files for a list of changes.  These files are distributed with
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
+ */
+
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GDK_H_INSIDE__) && !defined (GDK_COMPILATION)
+#error "Only <gdk/gdk.h> can be included directly."
+#endif
+
 #ifndef __GDK_DND_H__
 #define __GDK_DND_H__
 
@@ -47,29 +77,28 @@ struct _GdkDragContext {
 
   /*< public >*/
   
-  GdkDragProtocol protocol;
-  
-  gboolean is_source;
-  
-  GdkWindow *source_window;
-  GdkWindow *dest_window;
+  GdkDragProtocol GSEAL (protocol);
 
-  GList *targets;
-  GdkDragAction actions;
-  GdkDragAction suggested_action;
-  GdkDragAction action; 
+  gboolean GSEAL (is_source);
+  
+  GdkWindow *GSEAL (source_window);
+  GdkWindow *GSEAL (dest_window);
 
-  guint32 start_time;
+  GList *GSEAL (targets);
+  GdkDragAction GSEAL (actions);
+  GdkDragAction GSEAL (suggested_action);
+  GdkDragAction GSEAL (action);
+
+  guint32 GSEAL (start_time);
 
   /*< private >*/
   
-  gpointer windowing_data;
+  gpointer GSEAL (windowing_data);
 };
 
 struct _GdkDragContextClass {
   GObjectClass parent_class;
 
-  
 };
 
 /* Drag and Drop */
@@ -77,6 +106,11 @@ struct _GdkDragContextClass {
 GType            gdk_drag_context_get_type   (void) G_GNUC_CONST;
 GdkDragContext * gdk_drag_context_new        (void);
 
+GList           *gdk_drag_context_list_targets         (GdkDragContext *context);
+GdkDragAction    gdk_drag_context_get_actions          (GdkDragContext *context);
+GdkDragAction    gdk_drag_context_get_suggested_action (GdkDragContext *context);
+GdkDragAction    gdk_drag_context_get_action           (GdkDragContext *context);
+
 #ifndef GDK_DISABLE_DEPRECATED
 void             gdk_drag_context_ref        (GdkDragContext *context);
 void             gdk_drag_context_unref      (GdkDragContext *context);
@@ -100,9 +134,10 @@ GdkAtom          gdk_drag_get_selection (GdkDragContext   *context);
 GdkDragContext * gdk_drag_begin      (GdkWindow      *window,
 				      GList          *targets);
 
-guint32 gdk_drag_get_protocol_for_display (GdkDisplay       *display,
-					   guint32           xid,
-					   GdkDragProtocol  *protocol);
+GdkNativeWindow gdk_drag_get_protocol_for_display (GdkDisplay       *display,
+						   GdkNativeWindow   xid,
+						   GdkDragProtocol  *protocol);
+
 void    gdk_drag_find_window_for_screen   (GdkDragContext   *context,
 					   GdkWindow        *drag_window,
 					   GdkScreen        *screen,
@@ -112,8 +147,9 @@ void    gdk_drag_find_window_for_screen   (GdkDragContext   *context,
 					   GdkDragProtocol  *protocol);
 
 #ifndef GDK_MULTIHEAD_SAFE
-guint32 gdk_drag_get_protocol (guint32           xid,
-			       GdkDragProtocol  *protocol);
+GdkNativeWindow gdk_drag_get_protocol (GdkNativeWindow   xid,
+				       GdkDragProtocol  *protocol);
+
 void    gdk_drag_find_window  (GdkDragContext   *context,
 			       GdkWindow        *drag_window,
 			       gint              x_root,
