@@ -18,13 +18,25 @@
  * Boston, MA 02111-1307, USA.
  */
 
-
 #import "GdkQuartzView.h"
+#include "gdkregion.h"
+#include "gdkregion-generic.h"
 #include "gdkwindow-quartz.h"
 #include "gdkprivate-quartz.h"
 
 @implementation GdkQuartzView
 
+-(void)dealloc
+{
+  if (trackingRect)
+    {
+      [self removeTrackingRect:trackingRect];
+      trackingRect = 0;
+    }
+
+  [super dealloc];
+}
+
 -(void)setGdkWindow:(GdkWindow *)window
 {
   gdk_window = window;
@@ -35,6 +47,11 @@
   return gdk_window;
 }
 
+-(NSTrackingRectTag)trackingRect
+{
+  return trackingRect;
+}
+
 -(BOOL)isFlipped
 {
   return YES;
@@ -55,7 +72,8 @@
   GdkWindowObject *private = GDK_WINDOW_OBJECT (gdk_window);
   GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
   const NSRect *drawn_rects;
-  int count, i;
+  NSInteger count;
+  int i;
   GdkRegion *region;
 
   if (GDK_WINDOW_DESTROYED (gdk_window))
@@ -64,55 +82,32 @@
   if (!(private->event_mask & GDK_EXPOSURE_MASK))
     return;
 
-  /* For some reason, we occasionally get draw requests for zero sized rects
-   * at 0,0, just ignore those.
-   */
-  if (rect.origin.x == 0 && rect.origin.y == 0 &&
-      rect.size.width == 0 && rect.size.height == 0)
+  if (NSEqualRects (rect, NSZeroRect))
+    return;
+
+  /* Clear our own bookkeeping of regions that need display */
+  if (impl->needs_display_region)
     {
-      return;
+      gdk_region_destroy (impl->needs_display_region);
+      impl->needs_display_region = NULL;
     }
 
-  GDK_QUARTZ_ALLOC_POOL;
-
   [self getRectsBeingDrawn:&drawn_rects count:&count];
-
   region = gdk_region_new ();
-
+  
   for (i = 0; i < count; i++)
     {
       gdk_rect.x = drawn_rects[i].origin.x;
       gdk_rect.y = drawn_rects[i].origin.y;
       gdk_rect.width = drawn_rects[i].size.width;
       gdk_rect.height = drawn_rects[i].size.height;
-
+      
       gdk_region_union_with_rect (region, &gdk_rect);
     }
 
-  if (!gdk_region_empty (region))
-    {
-      GdkEvent event;
-      
-      gdk_rect.x = rect.origin.x;
-      gdk_rect.y = rect.origin.y;
-      gdk_rect.width = rect.size.width;
-      gdk_rect.height = rect.size.height;
-      
-      event.expose.type = GDK_EXPOSE;
-      event.expose.window = g_object_ref (gdk_window);
-      event.expose.send_event = FALSE;
-      event.expose.count = 0;
-      event.expose.region = region;
-      event.expose.area = gdk_rect;
-      
-      impl->in_paint_rect_count++;
-
-      (*_gdk_event_func) (&event, _gdk_event_data);
-
-      impl->in_paint_rect_count--;
-
-      g_object_unref (gdk_window);
-    }
+  impl->in_paint_rect_count++;
+  _gdk_window_process_updates_recurse (gdk_window, region);
+  impl->in_paint_rect_count--;
 
   gdk_region_destroy (region);
 
@@ -121,8 +116,6 @@
       [[self window] invalidateShadow];
       needsInvalidateShadow = NO;
     }
-
-  GDK_QUARTZ_RELEASE_POOL;
 }
 
 -(void)setNeedsInvalidateShadow:(BOOL)invalidate
@@ -139,6 +132,9 @@
   GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
   NSRect rect;
 
+  if (!impl->toplevel)
+    return;
+
   if (trackingRect)
     {
       [self removeTrackingRect:trackingRect];
@@ -157,6 +153,15 @@
                                  owner:self
                               userData:nil
                           assumeInside:NO];
+
+  if (NSPointInRect ([[self window] convertScreenToBase:[NSEvent mouseLocation]], rect))
+    {
+      /* When a new window (and thus view) has been created, and the mouse
+       * is in the window area, we will not receive an NSMouseEntered
+       * event.  Therefore, we synthesize an enter notify event manually.
+       */
+      _gdk_quartz_events_send_enter_notify_event (gdk_window);
+    }
 }
 
 -(void)viewDidMoveToWindow
