@@ -31,7 +31,7 @@
  * be in x11/gdkinput-x11.c.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include "gdkdisplay.h"
 #include "gdkinput.h"
@@ -73,7 +73,7 @@ gdk_device_get_type (void)
 
   if (!object_type)
     {
-      static const GTypeInfo object_info =
+      const GTypeInfo object_info =
 	{
 	  sizeof (GdkDeviceClass),
 	  (GBaseInitFunc) NULL,
@@ -109,6 +109,38 @@ gdk_display_list_devices (GdkDisplay *dpy)
   return _gdk_input_devices;
 }
 
+G_CONST_RETURN gchar *
+gdk_device_get_name (GdkDevice *device)
+{
+  g_return_val_if_fail (GDK_IS_DEVICE (device), NULL);
+
+  return device->name;
+}
+
+GdkInputSource
+gdk_device_get_source (GdkDevice *device)
+{
+  g_return_val_if_fail (GDK_IS_DEVICE (device), 0);
+
+  return device->source;
+}
+
+GdkInputMode
+gdk_device_get_mode (GdkDevice *device)
+{
+  g_return_val_if_fail (GDK_IS_DEVICE (device), 0);
+
+  return device->mode;
+}
+
+gboolean
+gdk_device_get_has_cursor (GdkDevice *device)
+{
+  g_return_val_if_fail (GDK_IS_DEVICE (device), FALSE);
+
+  return device->has_cursor;
+}
+
 void
 gdk_device_set_source (GdkDevice      *device,
 		       GdkInputSource  source)
@@ -118,6 +150,26 @@ gdk_device_set_source (GdkDevice      *device,
   device->source = source;
 }
 
+void
+gdk_device_get_key (GdkDevice       *device,
+                    guint            index,
+                    guint           *keyval,
+                    GdkModifierType *modifiers)
+{
+  g_return__if_fail (GDK_IS_DEVICE (device));
+  g_return_if_fail (index < device->num_keys);
+
+  if (!device->keys[index].keyval &&
+      !device->keys[index].modifiers)
+    return;
+
+  if (keyval)
+    *keyval = device->keys[index].keyval;
+
+  if (modifiers)
+    *modifiers = device->keys[index].modifiers;
+}
+
 void
 gdk_device_set_key (GdkDevice      *device,
 		    guint           index,
@@ -131,6 +183,16 @@ gdk_device_set_key (GdkDevice      *device,
   device->keys[index].modifiers = modifiers;
 }
 
+GdkAxisUse
+gdk_device_get_axis_use (GdkDevice *device,
+                         guint      index)
+{
+  g_return_val_if_fail (GDK_IS_DEVICE (device), GDK_AXIS_IGNORE);
+  g_return_val_if_fail (index < device->num_axes, GDK_AXIS_IGNORE);
+
+  return device->axes[index].use;
+}
+
 void
 gdk_device_set_axis_use (GdkDevice   *device,
 			 guint        index,
@@ -310,6 +372,32 @@ _gdk_input_window_destroy (GdkWindow *window)
   g_free(input_window);
 }
 
+void
+_gdk_input_crossing_event (GdkWindow *window,
+			   gboolean enter)
+{
+  if (enter)
+    {
+#if 0 /* No idea what to do... */
+      GdkWindowObject *priv = (GdkWindowObject *)window;
+      GdkInputWindow *input_window;
+      gint root_x, root_y;
+#if 0
+      gdk_input_check_proximity(display);
+#endif
+      input_window = priv->input_window;
+      if (input_window != NULL)
+	{
+	  _gdk_input_get_root_relative_geometry (window, &root_x, &root_y);
+	  input_window->root_x = root_x;
+	  input_window->root_y = root_y;
+	}
+#endif
+    }
+  else
+    _gdk_input_ignore_core = FALSE;
+}
+
 void
 _gdk_input_exit (void)
 {
