@@ -20,6 +20,7 @@
 /**
  * SECTION:gtkactivatable
  * @Short_Description: An interface for activatable widgets
+ * @Title: GtkActivatable
  *
  * Activatable widgets can be connected to a #GtkAction and reflects
  * the state of its action. A #GtkActivatable can also provide feedback
@@ -275,13 +276,16 @@ gtk_activatable_get_type (void)
 {
   static GType activatable_type = 0;
 
-  if (!activatable_type)
+  if (!activatable_type) {
     activatable_type =
       g_type_register_static_simple (G_TYPE_INTERFACE, I_("GtkActivatable"),
 				     sizeof (GtkActivatableIface),
 				     (GClassInitFunc) gtk_activatable_class_init,
 				     0, NULL, 0);
 
+    g_type_interface_add_prerequisite (activatable_type, G_TYPE_OBJECT);
+  }
+
   return activatable_type;
 }
 
@@ -352,7 +356,7 @@ gtk_activatable_update (GtkActivatable *activatable,
 /**
  * gtk_activatable_sync_action_properties:
  * @activatable: a #GtkActivatable
- * @action: the related #GtkAction or %NULL
+ * @action: (allow-none): the related #GtkAction or %NULL
  *
  * This is called to update the activatable completely, this is called
  * internally when the #GtkActivatable::related-action property is set
@@ -444,12 +448,14 @@ gtk_activatable_do_set_related_action (GtkActivatable *activatable,
 	{
 	  g_signal_handlers_disconnect_by_func (prev_action, gtk_activatable_action_notify, activatable);
 	  
-	  _gtk_action_remove_from_proxy_list (prev_action, GTK_WIDGET (activatable));
+          /* Check the type so that actions can be activatable too. */
+          if (GTK_IS_WIDGET (activatable))
+            _gtk_action_remove_from_proxy_list (prev_action, GTK_WIDGET (activatable));
 	  
           /* Some apps are using the object data directly...
            * so continue to set it for a bit longer
            */
-          g_object_set_data (activatable, "gtk-action", NULL);
+          g_object_set_data (G_OBJECT (activatable), "gtk-action", NULL);
 
           /*
            * We don't want prev_action to be activated
@@ -476,9 +482,10 @@ gtk_activatable_do_set_related_action (GtkActivatable *activatable,
 
 	  g_signal_connect (G_OBJECT (action), "notify", G_CALLBACK (gtk_activatable_action_notify), activatable);
 
-	  _gtk_action_add_to_proxy_list (action, GTK_WIDGET (activatable));
+          if (GTK_IS_WIDGET (activatable))
+            _gtk_action_add_to_proxy_list (action, GTK_WIDGET (activatable));
 
-          g_object_set_data (activatable, "gtk-action", action);
+          g_object_set_data (G_OBJECT (activatable), "gtk-action", action);
 	}
     }
 }
